@@ -228,6 +228,11 @@ sub new {
                                       not $self->field_is_skipout($_);
                                     }
                                     $self->get_fields_of_type('list', 'uri')->@*],
+                      verbs     => [sort grep
+                                    {
+                                      not $self->field_is_skipout($_);
+                                    }
+                                    $self->get_fields_of_datatype(['verbatim', 'uri'])->@*],
                       vfields   => [sort grep
                                     {
                                       not $self->field_is_skipout($_);
@@ -1632,16 +1637,16 @@ sub generate_bblxml_schema {
   $writer->endTag();    # choice
   $writer->endTag();    # attribute
   $writer->endTag();    # optional
-  # dateparts may have a circa attributes
+  # dateparts may have a approximate attributes
   $writer->startTag('optional');
-  $writer->startTag('attribute', 'name' => 'startcirca');
+  $writer->startTag('attribute', 'name' => 'startapproximate');
   $writer->startTag('choice');
   $writer->dataElement('value', 'true');
   $writer->endTag();    # choice
   $writer->endTag();    # attribute
   $writer->endTag();    # optional
   $writer->startTag('optional');
-  $writer->startTag('attribute', 'name' => 'endcirca');
+  $writer->startTag('attribute', 'name' => 'endapproximate');
   $writer->startTag('choice');
   $writer->dataElement('value', 'true');
   $writer->endTag();    # choice
