@@ -356,6 +356,7 @@ sub extract_entries {
     # This is not so bad as they have a clean structure (see error.c in libbtparse)
     open my $tbe, '<', $tberr_name;
     while (<$tbe>) {
+      next if /overriding\sexisting\sdefinition\sof\smacro/; # ignore macro redefs
       if (/error:/) {
         chomp;
         biber_error("BibTeX subsystem: $_");
@@ -1164,9 +1165,9 @@ sub _datetime {
       $bibentry->set_field($datetype . 'datejulian', 1) if $CONFIG_DATE_PARSERS{start}->julian;
       $bibentry->set_field($datetype . 'enddatejulian', 1) if $CONFIG_DATE_PARSERS{end}->julian;
 
-      # Save circa information
-      $bibentry->set_field($datetype . 'datecirca', 1) if $CONFIG_DATE_PARSERS{start}->circa;
-      $bibentry->set_field($datetype . 'enddatecirca', 1) if $CONFIG_DATE_PARSERS{end}->circa;
+      # Save approximate information
+      $bibentry->set_field($datetype . 'dateapproximate', 1) if $CONFIG_DATE_PARSERS{start}->approximate;
+      $bibentry->set_field($datetype . 'enddateapproximate', 1) if $CONFIG_DATE_PARSERS{end}->approximate;
 
       # Save uncertain date information
       $bibentry->set_field($datetype . 'dateuncertain', 1) if $CONFIG_DATE_PARSERS{start}->uncertain;
@@ -1348,9 +1349,15 @@ sub cache_data {
       my $S = qr/$Srx/;
       foreach my $id (split(/$S/, $ids)) {
 
+        # Skip aliases which are this very key (deep recursion ...)
+        if (fc($id) eq fc($key)) {
+          biber_warn("BAD RECURSION! Entry alias '$id' is identical to the entry key, skipping ...");
+          next;
+        }
+
         # Skip aliases which are also real entry keys
         if ($section->has_everykey($id)) {
-          biber_warn("Citekey alias '$id' is also a real entry key, skipping ...");
+          biber_warn("Entry alias '$id' is also a real entry key, skipping ...");
           next;
         }
 
@@ -1358,13 +1365,13 @@ sub cache_data {
         if (exists($cache->{data}{citekey_aliases}{$id})) {
           my $otherid = $cache->{data}{citekey_aliases}{$id};
           if ($otherid ne $key) {
-            biber_warn("Citekey alias '$id' already has an alias '$otherid', skipping ...");
+            biber_warn("Entry alias '$id' already has an alias '$otherid', skipping ...");
           }
         }
         else {
           $cache->{data}{citekey_aliases}{$id} = $key;
           if ($logger->is_debug()) {# performance tune
-            $logger->debug("Citekey '$id' is an alias for citekey '$key'");
+            $logger->debug("Entry alias '$id' is an alias for citekey '$key'");
           }
         }
       }
@@ -1433,6 +1440,10 @@ sub preprocess_file {
   # just in case there was a BOM so we can delete it as it makes T::B complain
   # Might fail due to encountering characters invalid in the encoding so trap and die gracefully
   my $benc = Biber::Config->getoption('input_encoding');
+  if ($benc eq 'ascii') {
+    $logger->info("Reading ascii input as UTF-8");
+    $benc = 'UTF-8';
+  }
   my $buf;
   unless (eval{$buf = NFD(File::Slurper::read_text($filename, $benc))}) {# Unicode NFD boundary
     biber_error("Data file '$filename' cannot be read in encoding '$benc': $@");
@@ -1510,9 +1521,12 @@ sub parse_decode {
     }
   }
 
-  # We will read in the same .bib again later to do the real parsing
-  # and since macro defs are global, need to reset them to avoid redef warnings
-  Text::BibTeX::delete_all_macros();
+  # (Re-)define the old BibTeX month macros to what biblatex wants unless user stops this
+  unless (Biber::Config->getoption('nostdmacros')) {
+    foreach my $mon (keys %MONTHS) {
+      Text::BibTeX::add_macro_text($mon, $MONTHS{$mon});
+    }
+  }
 
   return $lbuf;
 }
