@@ -128,9 +128,9 @@ sub set_output_entry {
         push @namelist, 'useprefix=' . Biber::Utils::map_boolean($names->get_useprefix, 'tostring');
       }
 
-      # Namelist scope sortnamekeyscheme
-      if (my $snks = $names->get_sortnamekeyscheme) {
-        push @namelist, "sortnamekeyscheme=$snks";
+      # Namelist scope sortingnamekeytemplatename
+      if (my $snks = $names->get_sortingnamekeytemplatename) {
+        push @namelist, "sortingnamekeytemplatename=$snks";
       }
 
       # Now add all names to accumulator
@@ -187,11 +187,13 @@ sub set_output_entry {
   }
 
   # If CROSSREF and XDATA have been resolved, don't output them
-  if (Biber::Config->getoption('output_resolve')) {
-    if ( my $cr = $be->get_field('crossref') ) {
+  if (Biber::Config->getoption('output_resolve_crossrefs')) {
+    if ($be->get_field('crossref')) {
       $be->del_field('crossref');
     }
-    if ( my $xd = $be->get_field('xdata') ) {
+  }
+  if (Biber::Config->getoption('output_resolve_xdata')) {
+    if ($be->get_field('xdata')) {
       $be->del_field('xdata');
     }
   }
@@ -348,7 +350,14 @@ sub output {
   }
 
   # Bibtex output uses just one special section, always sorted by global sorting spec
-  foreach my $key ($Biber::MASTER->sortlists->get_list(99999, Biber::Config->getblxoption('sortscheme') . '/global/', 'entry', Biber::Config->getblxoption('sortscheme'), 'global', '')->get_keys) {
+  foreach my $key ($Biber::MASTER->datalists->get_lists_by_attrs(section => 99999,
+                                                                 name => Biber::Config->getblxoption('sortingtemplatename') . '/global//global/global',
+                                                                 type => 'entry',
+                                                                 sortingtemplatename => Biber::Config->getblxoption('sortingtemplatename'),
+                                                                 sortingnamekeytemplatename => 'global',
+                                                                 labelprefix => '',
+                                                                 uniquenametemplatename => 'global',
+                                                                 labelalphanametemplatename => 'global')->[0]->get_keys->@*) {
     out($target, ${$data->{ENTRIES}{99999}{index}{$key}});
   }
 
@@ -493,39 +502,39 @@ sub construct_datetime {
                   'autumn' => 23,
                   'winter' => 24 );
 
-  # Did the date fields come from interpreting an EDTF 5.2.2 unspecified date?
-  # If so, do the reverse of Biber::Utils::parse_date_edtf_unspecified()
+  # Did the date fields come from interpreting an ISO8601-2:2016 unspecified date?
+  # If so, do the reverse of Biber::Utils::parse_date_unspecified()
   if (my $unspec = $be->get_field("${d}dateunspecified")) {
 
-    # 1990/1999 -> 199u
+    # 1990/1999 -> 199X
     if ($unspec eq 'yearindecade') {
       my ($decade) = $be->get_field("${d}year") =~ m/^(\d+)\d$/;
-      $overridey = "${decade}u";
+      $overridey = "${decade}X";
       $be->del_field("${d}endyear");
     }
-    # 1900/1999 -> 19uu
+    # 1900/1999 -> 19XX
     elsif ($unspec eq 'yearincentury') {
       my ($century) = $be->get_field("${d}year") =~ m/^(\d+)\d\d$/;
-      $overridey = "${century}uu";
+      $overridey = "${century}XX";
       $be->del_field("${d}endyear");
     }
-    # 1999-01/1999-12 => 1999-uu
+    # 1999-01/1999-12 => 1999-XX
     elsif ($unspec eq 'monthinyear') {
-      $overridem = 'uu';
+      $overridem = 'XX';
       $be->del_field("${d}endyear");
       $be->del_field("${d}endmonth");
     }
-    # 1999-01-01/1999-01-31 -> 1999-01-uu
+    # 1999-01-01/1999-01-31 -> 1999-01-XX
     elsif ($unspec eq 'dayinmonth') {
-      $overrided = 'uu';
+      $overrided = 'XX';
       $be->del_field("${d}endyear");
       $be->del_field("${d}endmonth");
       $be->del_field("${d}endday");
     }
-    # 1999-01-01/1999-12-31 -> 1999-uu-uu
+    # 1999-01-01/1999-12-31 -> 1999-XX-XX
     elsif ($unspec eq 'dayinyear') {
-      $overridem = 'uu';
-      $overrided = 'uu';
+      $overridem = 'XX';
+      $overrided = 'XX';
       $be->del_field("${d}endyear");
       $be->del_field("${d}endmonth");
       $be->del_field("${d}endday");
@@ -557,14 +566,21 @@ sub construct_datetime {
       $be->del_field("${d}day");
     }
 
-    # Uncertain start date
-    if ($be->get_field("${d}dateuncertain")) {
-      $datestring .= '?';
+    # Uncertain and approximate start date
+    if ($be->get_field("${d}dateuncertain") and
+        $be->get_field("${d}dateapproximate")) {
+      $datestring .= '%';
     }
+    else {
+      # Uncertain start date
+      if ($be->get_field("${d}dateuncertain")) {
+        $datestring .= '?';
+      }
 
-    # Circa start date
-    if ($be->get_field("${d}datecirca")) {
-      $datestring .= '~';
+      # Approximate start date
+      if ($be->get_field("${d}dateapproximate")) {
+        $datestring .= '~';
+      }
     }
 
     # If start hour, there must be minute and second
@@ -606,14 +622,21 @@ sub construct_datetime {
         $be->del_field("${d}endday");
       }
 
-      # Uncertain start date
-      if ($be->get_field("${d}enddateuncertain")) {
-        $datestring .= '?';
+      # Uncertain and approximate end date
+      if ($be->get_field("${d}enddateuncertain") and
+          $be->get_field("${d}enddateapproximate")) {
+        $datestring .= '%';
       }
+      else {
+        # Uncertain end date
+        if ($be->get_field("${d}enddateuncertain")) {
+          $datestring .= '?';
+        }
 
-      # Circa start date
-      if ($be->get_field("${d}enddatecirca")) {
-        $datestring .= '~';
+        # Approximate end date
+        if ($be->get_field("${d}enddateapproximate")) {
+          $datestring .= '~';
+        }
       }
 
       # If end hour, there must be minute and second
@@ -653,7 +676,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2016 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2017 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
