@@ -55,7 +55,7 @@ our @EXPORT = qw{ locate_biber_file makenamesid makenameid stringify_hash
   bcp472locale rangelen match_indices process_comment map_boolean
   parse_range parse_range_alt maploopreplace get_transliterator
   call_transliterator normalise_string_bblxml gen_initials join_name_parts
-  split_xsv edtf_monthday tzformat};
+  split_xsv date_monthday tzformat expand_option};
 
 =head1 FUNCTIONS
 
@@ -76,6 +76,7 @@ sub locate_biber_file {
   my $filename = shift;
   my $filenamepath = $filename; # default if nothing else below applies
   my $foundfile;
+
   # If input_directory is set, perhaps the file can be found there so
   # construct a path to test later
   if (my $indir = Biber::Config->getoption('input_directory')) {
@@ -88,18 +89,17 @@ sub locate_biber_file {
   }
 
   # Filename is absolute
-  if (File::Spec->file_name_is_absolute($filename) and -e $filename) {
-    return $filename;
+  if (File::Spec->file_name_is_absolute($filename) and my $f = file_exist_check($filename)) {
+    return $f;
   }
 
   # File is input_directory or output_directory
-  if (defined($foundfile) and -e $foundfile) {
-    return $foundfile;
+  if (defined($foundfile) and my $f = file_exist_check($foundfile)) {
+    return $f;
   }
 
-  # File is relative to cwd
-  if (-e $filename) {
-    return $filename;
+  if (my $f = file_exist_check($filename)) {
+    return $f;
   }
 
   # File is where control file lives
@@ -114,7 +114,9 @@ sub locate_biber_file {
 
     my $path = "$ctlvolume$ctldir$filename";
 
-    return $path if -e $path;
+    if (my $f = file_exist_check($path)) {
+      return $f;
+    }
   }
 
   # File is in kpse path
@@ -140,7 +142,8 @@ sub locate_biber_file {
       chomp $found;
       $found =~ s/\cM\z//xms; # kpsewhich in cygwin sometimes returns ^M at the end
       # filename can be UTF-8 and run3() isn't clever with UTF-8
-      return decode_utf8($found);
+      my $f = file_exist_check(decode_utf8($found));
+      return $f;
     }
     else {
       if ($logger->is_debug()) {# performance tune
@@ -151,6 +154,32 @@ sub locate_biber_file {
   return undef;
 }
 
+=head2 
+
+  Check existence of NFC/NFD file variants and return correct one.
+  Account for windows file encodings
+
+=cut
+
+sub file_exist_check {
+  my $filename = shift;
+  if ($^O =~ /Win/) {
+    require Win32;
+    my $f = Win32::GetANSIPathName($filename);
+    return $f if -e $f;
+  }
+  else {
+    if (-e NFC($filename)) {
+      return NFC($filename);
+    }
+    if (-e NFD($filename)) {
+      return NFD($filename);
+    }
+  }
+
+  return undef;
+}
+
 =head2 biber_warn
 
     Wrapper around various warnings bits and pieces
@@ -308,8 +337,6 @@ sub normalise_string_label {
 
 Removes LaTeX macros, and all punctuation, symbols, separators and control characters,
 as well as leading and trailing whitespace for sorting strings.
-It also decodes LaTeX character macros into Unicode as this is always safe when
-normalising strings for sorting since they don't appear in the output.
 
 =cut
 
@@ -321,7 +348,12 @@ sub normalise_string_sort {
   $str = strip_nosort($str, $fieldname);
   # Then replace ties with spaces or they will be lost
   $str =~ s/([^\\])~/$1 /g; # Foo~Bar -> Foo Bar
-  return normalise_string_common($str);
+  # Don't use normalise_string_common() as this strips out things needed for sorting
+  $str =~ s/\\[A-Za-z]+//g;        # remove latex macros (assuming they have only ASCII letters)
+  $str =~ s/[\{\}\p{C}]+//g;       # remove embedded braces and control
+  $str =~ s/^\s+|\s+$//g;          # Remove leading and trailing spaces
+  $str =~ s/\s+/ /g;               # collapse spaces
+  return $str;
 }
 
 =head2 normalise_string_bblxml
@@ -334,7 +366,7 @@ sub normalise_string_bblxml {
   my $str = shift;
   return '' unless $str; # Sanitise missing data
   $str =~ s/\\[A-Za-z]+//g; # remove latex macros (assuming they have only ASCII letters)
-  $str =~ s/\{([^{}]+)\}/$1/g; # remove pointless braces
+  $str =~ s/\{([^\{\}]+)\}/$1/g; # remove pointless braces
   $str =~ s/~/ /g; # replace ties with spaces
   return $str;
 }
@@ -364,7 +396,7 @@ sub normalise_string {
 sub normalise_string_common {
   my $str = shift;
   $str =~ s/\\[A-Za-z]+//g;        # remove latex macros (assuming they have only ASCII letters)
-  $str =~ s/[\p{P}\p{S}\p{C}]+//g; # remove punctuation, symbols, separator and control
+  $str =~ s/[\p{P}\p{S}\p{C}]+//g; # remove punctuation, symbols and control
   $str =~ s/^\s+|\s+$//g;          # Remove leading and trailing spaces
   $str =~ s/\s+/ /g;               # collapse spaces
   return $str;
@@ -390,7 +422,7 @@ sub normalise_string_hash {
   return '' unless $str; # Sanitise missing data
   $str =~ s/\\(\p{L}+)\s*/$1:/g; # remove tex macros
   $str =~ s/\\([^\p{L}])\s*/ord($1).':'/ge; # remove accent macros like \"a
-  $str =~ s/[{}~\.\s]+//g; # Remove brackes, ties, dots, spaces
+  $str =~ s/[\{\}~\.\s]+//g; # Remove brackes, ties, dots, spaces
   return $str;
 }
 
@@ -757,7 +789,7 @@ sub join_name {
 
 =head2 filter_entry_options
 
-    Process any per_entry option transformations which are necessary
+    Process any per_entry option transformations which are necessary on output
 
 =cut
 
@@ -958,48 +990,52 @@ sub process_entry_options {
   return unless $options;       # Just in case it's null
   foreach ($options->@*) {
     s/\s+=\s+/=/g; # get rid of spaces around any "="
-    m/^([^=]+)(=?)(.+)?$/;
-    if ($2) {
-      if ($CONFIG_OPTTYPE_BIBLATEX{lc($1)} and
-          $CONFIG_OPTTYPE_BIBLATEX{lc($1)} eq 'boolean') {
-        _expand_option($1, map_boolean($3, 'tonum'), $citekey);
-      }
-      else {
-        _expand_option($1, $3, $citekey);
-      }
+    m/^([^=]+)=?(.+)?$/;
+    my $val = $2 // 1; # bare options are just boolean numerals
+    if ($CONFIG_OPTTYPE_BIBLATEX{lc($1)} and
+        $CONFIG_OPTTYPE_BIBLATEX{lc($1)} eq 'boolean') {
+      $val = map_boolean($val, 'tonum');
     }
-    else {
-      _expand_option($1, 1, $citekey);
+    my $oo = expand_option($1, $val, $CONFIG_BIBLATEX_ENTRY_OPTIONS{lc($1)}->{INPUT});
+
+    foreach my $o ($oo->@*) {
+      Biber::Config->setblxoption($o->[0], $o->[1], 'ENTRY', $citekey);
     }
   }
   return;
 }
 
-sub _expand_option {
-  my ($opt, $val, $citekey) = @_;
-  my $cfopt = $CONFIG_BIBLATEX_ENTRY_OPTIONS{lc($opt)}{INPUT};
+=head2 expand_option
+
+    Expand option such as meta-options coming from biblatex
+
+=cut
+
+sub expand_option {
+  my ($opt, $val, $cfopt) = @_;
+  my $outopts;
   # Standard option
   if (not defined($cfopt)) {
-    Biber::Config->setblxoption($opt, $val, 'ENTRY', $citekey);
+    push $outopts->@*, [$opt, $val];
   }
-  # Set all split options to same value as parent
+  # Set all split options
   elsif (ref($cfopt) eq 'ARRAY') {
     foreach my $k ($cfopt->@*) {
-      Biber::Config->setblxoption($k, $val, 'ENTRY', $citekey);
+      push $outopts->@*, [$k, $val];
     }
   }
   # Specify values per all splits
   elsif (ref($cfopt) eq 'HASH') {
     foreach my $k (keys $cfopt->%*) {
-      Biber::Config->setblxoption($k, $cfopt->{$k}, 'ENTRY', $citekey);
+      push $outopts->@*, [$k, $cfopt->{$k}];
     }
   }
-  return;
+  return $outopts;
 }
 
 =head2 parse_date_range
 
-  Parse of EDTF date range
+  Parse of ISO8601 date range
   Returns two-element array ref: [start DT object, end DT object]
 
 =cut
@@ -1008,49 +1044,45 @@ sub parse_date_range {
   my ($bibentry, $datetype, $datestring) = @_;
   my ($sd, $sep, $ed) = $datestring =~ m|^([^/]+)?(/)?([^/]+)?$|;
   my $unspec;
-  if ($sd =~ /u/ and $sd !~ /unknown/) {# EDTF 5.2.2 Unspecified format but not 5.2.3
-    ($sd, $sep, $ed, $unspec) = parse_date_edtf_unspecified($sd);
+  if ($sd =~ /X/) {# ISO8601-2 4.3 unspecified format
+    ($sd, $sep, $ed, $unspec) = parse_date_unspecified($sd);
   }
   # Set start date unknown flag
-  if ($sd) {
-    if (fc($sd) eq fc('unknown') or fc($sd) eq fc('*')) {
-      $bibentry->set_field($datetype . 'dateunknown',1);
-    }
+  if ($sep and not $sd) {
+    $bibentry->set_field($datetype . 'dateunknown',1);
   }
   # Set end date unknown flag
-  if ($ed) {
-    if (fc($ed) eq fc('unknown') or fc($ed) eq fc('*')) {
-      $bibentry->set_field($datetype . 'enddateunknown',1);
-    }
+  if ($sep and not $ed) {
+    $bibentry->set_field($datetype . 'enddateunknown',1);
   }
   return (parse_date_start($sd), parse_date_end($ed), $sep, $unspec);
 }
 
-=head2 parse_date_edtf_unspecified
+=head2 parse_date_unspecified
 
-  Parse of EDTF 5.2.2 Unspecified format into date range
+  Parse of ISO8601-2:2016 4.3 unspecified format into date range
   Returns range plus specification of granularity of unspecified
 
 =cut
 
-sub parse_date_edtf_unspecified {
+sub parse_date_unspecified {
   my $d = shift;
 
-  # 199u -> 1990/1999
-  if ($d =~ m/^(\d{3})u$/) {
+  # 199X -> 1990/1999
+  if ($d =~ m/^(\d{3})X$/) {
     return ("${1}0", '/', "${1}9", 'yearindecade');
   }
-  # 19uu -> 1900/1999
-  elsif ($d =~ m/^(\d{2})uu$/) {
+  # 19XX -> 1900/1999
+  elsif ($d =~ m/^(\d{2})XX$/) {
     return ("${1}00", '/', "${1}99", 'yearincentury');
   }
-  # 1999-uu -> 1999-01/1999-12
-  elsif ($d =~ m/^(\d{4})\p{Dash}uu$/) {
+  # 1999-XX -> 1999-01/1999-12
+  elsif ($d =~ m/^(\d{4})\p{Dash}XX$/) {
     return ("${1}-01", '/', "${1}-12", 'monthinyear');
   }
-  # 1999-01-uu -> 1999-01-01/1999-01-31
+  # 1999-01-XX -> 1999-01-01/1999-01-31
   # (understands different months and leap years)
-  elsif ($d =~ m/^(\d{4})\p{Dash}(\d{2})\p{Dash}uu$/) {
+  elsif ($d =~ m/^(\d{4})\p{Dash}(\d{2})\p{Dash}XX$/) {
 
     sub leapyear {
       my $year = shift;
@@ -1070,8 +1102,8 @@ sub parse_date_edtf_unspecified {
 
     return ("${1}-${2}-01", '/', "${1}-${2}-" . $monthdays{$2}, 'dayinmonth');
   }
-  # 1999-uu-uu -> 1999-01-01/1999-12-31
-  elsif ($d =~ m/^(\d{4})\p{Dash}uu\p{Dash}uu$/) {
+  # 1999-XX-XX -> 1999-01-01/1999-12-31
+  elsif ($d =~ m/^(\d{4})\p{Dash}XX\p{Dash}XX$/) {
     return ("${1}-01-01", '/', "${1}-12-31", 'dayinyear');
   }
 }
@@ -1108,9 +1140,7 @@ sub parse_date {
   # Must do this to make sure meta-information from sub-class Biber::Date::Format is reset
   $obj->init();
   return 0 unless $string;
-  return 0 if $string eq 'unknown'; # EDTF 5.2.3
-  return 0 if $string eq '*';       # ISO8601-2 equivalent for "unknown"
-  return 0 if $string eq 'open';    # EDTF 5.2.3
+  return 0 if $string eq '..';    # ISO8601-2 4.4 (open date)
 
   my $dt = eval {$obj->parse_datetime($string)};
 
@@ -1145,13 +1175,13 @@ sub parse_date {
   return $dt;
 }
 
-=head2 edtf_monthday
+=head2 date_monthday
 
-  Force month/day to EDTF format with leading zero
+  Force month/day to ISO8601-2:2016 format with leading zero
 
 =cut
 
-sub edtf_monthday {
+sub date_monthday {
   my $md = shift;
   return $md ? sprintf('%.2d', $md) : undef;
 }
@@ -1384,20 +1414,28 @@ sub maploopreplace {
 
 sub get_transliterator {
   my ($target, $from, $to) = map {lc} @_;
-  my @valid_from = ('iast');
-  my @valid_to   = ('devanagari');
+  my @valid_from = ('iast', 'russian');
+  my @valid_to   = ('devanagari', 'ala-lc', 'bgn/pcgn-standard');
   unless (first {$from eq $_} @valid_from and
           first {$to eq $_} @valid_to) {
     biber_warn("Invalid transliteration from/to pair ($from/$to)");
   }
   require Lingua::Translit;
+  if ($logger->is_debug()) {# performance tune
+    $logger->debug("Using '$from -> $to' transliteration for sorting '$target'");
+  }
+
   # List pairs explicitly as we don't expect there to be to many of these ever
   if ($from eq 'iast' and $to eq 'devanagari') {
-    if ($logger->is_debug()) {# performance tune
-      $logger->debug("Using 'iast -> devanagari' transliteration for sorting '$target'");
-    }
     return new Lingua::Translit('IAST Devanagari');
   }
+  elsif ($from eq 'russian' and $to eq 'ala-lc') {
+    return new Lingua::Translit('ALA-LC RUS');
+  }
+  elsif ($from eq 'russian' and $to eq 'bgn/pcgn-standard') {
+    return new Lingua::Translit('BGN/PCGN RUS Standard');
+  }
+
   return undef;
 }
 
@@ -1504,7 +1542,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2016 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2017 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
